@@ -31,7 +31,7 @@ from cognifly.utils.tcp_video_interface import TCPVideoInterface
 from cognifly.utils.ip_tools import extract_ip
 from cognifly.utils.pid import PID
 from cognifly.utils.filters import Simple1DKalman, Simple1DExponentialAverage
-from cognifly.utils.functions import clip
+from cognifly.utils.functions import clip, smallest_angle_diff_rad
 
 SWITCH_MODE_CHR = ord('m')
 FORWARD_CHR = ord('8')  # curses.KEY_UP
@@ -126,16 +126,6 @@ class SignalTracer:
         self.file.close()
 
 
-def smallest_angle_diff_rad(theta_new, theta_old):
-    """
-    computes theta_new - theta_old (radian)
-    """
-    diff = theta_new - theta_old
-    mod_p = diff % (2 * np.pi)
-    mod_n = diff % (-2 * np.pi)
-    return mod_p if abs(mod_p) < abs(mod_n) else mod_n
-
-
 class CogniflyController:
     def __init__(self,
                  network=True,
@@ -853,11 +843,12 @@ class CogniflyController:
                                 screen.clrtoeol()
 
                                 self.debug_flags = board.process_armingDisableFlags(board.CONFIG['armingDisableFlags'])
-                                cam_err, cam_exp = self.tcp_video_int.get_camera_error()
+                                cam_err, cam_exp, cam_trace = self.tcp_video_int.get_camera_error()
                                 if cam_err:
                                     self.debug_flags.append("CAMERA_ERROR")
                                     screen.addstr(12, 0, f"CAMERA ERROR: {cam_exp}")
                                     screen.clrtoeol()
+                                    # raise Exception(cam_trace)
                                 screen.addstr(5, 50, "armingDisableFlags: {}".format(self.debug_flags))
                                 screen.clrtoeol()
 
