@@ -43,7 +43,8 @@ class Cognifly:
                  recv_port_video=8990,
                  wait_for_first_obs=True,
                  wait_for_first_obs_sleep_duration=0.1,
-                 wait_ack_duration=0.2):
+                 wait_ack_duration=0.2,
+                 gui=True):
         """
         Args:
             drone_hostname: string: name of the drone (can be an ip address).
@@ -54,7 +55,9 @@ class Cognifly:
             wait_for_first_obs_sleep_duration: float: used only if wait_for_first_obs is True
             wait_ack_duration: float: If > 0.0, the framework will wait this number of seconds for an acknowledgement before resending.
                 If <= 0.0, the resending feature is disabled.
+            gui: bool: if True, a gui window will pop when the object is created (emergency disarm, battery, streaming).
         """
+        self.gui = gui
         self.drone_hostname = drone_hostname
         self.send_port = send_port
         self.recv_port = recv_port
@@ -92,6 +95,12 @@ class Cognifly:
         self.__last_i_obs = 0
         self.__wait_done = False
 
+        self._display_lock = Lock()
+        self.__display = False
+
+        self._batt_lock = Lock()
+        self.__batt_str = "OK"
+
         self._listener_thread = Thread(target=self.__listener_thread)
         self._listener_thread.setDaemon(True)  # thread will be terminated at exit
         self._listener_thread.start()
@@ -104,6 +113,53 @@ class Cognifly:
 
         self.reset()
 
+        if self.gui:
+            self._gui_thread = Thread(target=self.__gui_thread, args=(self.drone_hostname, ))
+            self._gui_thread.setDaemon(True)  # thread will be terminated at exit
+            self._gui_thread.start()
+        else:
+            self._gui_thread = None
+
+    def __gui_thread(self, drone_name):
+        """
+        Thread in charge of managing the Graphical User Interface
+        """
+        import PySimpleGUI as sg
+        import cv2
+        sg.theme('BluePurple')
+        layout = [[sg.Text('Battery:'), sg.Text('OK', size=(15, 1), key='-batt-')],
+                  [sg.Button('DISARM')],
+                  [sg.Image(filename='', key='-image-')]]
+        window = sg.Window(drone_name,
+                           layout,
+                           keep_on_top=True,
+                           enable_close_attempted_event=True)
+        image_elem = window['-image-']
+        batt_elem = window['-batt-']
+        try:
+            while True:  # Event Loop
+                with self._display_lock:
+                    # check whether the display is on:
+                    if self.__display:
+                        event, values = window.read(0)
+                        with self.tcp_video_int.condition_in:
+                            self.tcp_video_int.condition_in.wait()  # wait for new frame
+                            frame, _ = self.tcp_video_int.get_last_image()
+                        if frame is not None:  # should not be None but let's be cautious
+                            imgbytes = cv2.imencode('.png', frame)[1].tobytes()  # ditto
+                            image_elem.update(data=imgbytes)
+                    else:
+                        event, values = window.read(200)  # 5 Hz to check change of self.__display
+                        image_elem.update(data=b'')
+                if event == 'DISARM':
+                    self.disarm()  # thread-safe because the whole self.send procedure uses self._lock
+                if event == sg.WINDOW_CLOSE_ATTEMPTED_EVENT and sg.popup_yes_no('Do you really want to exit?', keep_on_top=True) == 'Yes':
+                    break
+                with self._batt_lock:
+                    batt_elem.update(self.__batt_str)
+        finally:
+            window.close()
+
     def __listener_thread(self):
         """
         Thread in charge of receiving UDP messages.
@@ -134,8 +190,8 @@ class Cognifly:
                         self.__wait_done = False
                     self._lock.release()
                 elif m[0] == "BBT":  # bad battery state
-                    pass
-                    # print(f"Low battery: {m[1]}V, the drone will soon refuse to move.")
+                    with self._batt_lock:
+                        self.__batt_str = "LOW!"
 
     def __resender_thread(self):
         """
@@ -155,8 +211,8 @@ class Cognifly:
         """
         Send an UDP message to the drone.
         Args:
-            msg_type: string in ["RES", ACT"]
-            msg: tuple of the form (str:command, ...:args)
+            msg_type: string in ["RES", ACT"].
+            msg: tuple of the form (str:command, ...:args).
         """
         self._lock.acquire()
         self.__last_sent_id += 1
@@ -164,8 +220,8 @@ class Cognifly:
         data = (msg_type, self.__last_sent_id, msg) if msg is not None else (msg_type, self.__last_sent_id)
         data_s = pkl.dumps(data)
         self.__last_sent = data_s
-        self._lock.release()
         self.udp_int.send(data_s)
+        self._lock.release()
 
     def sleep_until_done(self, max_duration, granularity=0.05):
         """
@@ -524,7 +580,7 @@ class Cognifly:
 
     # Methods common to both APIs:
 
-    def streamon(self, resolution="VGA", fps=5, display=False, wait_first_frame=True):
+    def streamon(self, resolution="VGA", fps=24, display=False, wait_first_frame=True):
         """
         Starts camera streaming.
         CAUTION: This will slow the frequency of the onboard controller down and may make the drone unstable!
@@ -544,8 +600,11 @@ class Cognifly:
         self.send(msg_type="ST1", msg=(self.local_ip, self.recv_port_video, resolution, fps))
         if wait_first_frame:
             _ = self.get_frame(wait_new_frame=True)
+        self._display_lock.acquire()
+        self.__display = display
+        self._display_lock.release()
 
-    def stream(self, resolution="VGA", fps=5):
+    def stream(self, resolution="VGA", fps=24):
         """
         Alias for streamon(resolution, fps=5, display=True)
         """
@@ -570,14 +629,16 @@ class Cognifly:
             if cond:
                 time.sleep(sleep_between_trials)
         if cond1:
-            raise TimeoutException("Could not retrieve frame from stream.")
-        else:
-            return im
+            raise TimeoutError("Could not retrieve frame from stream.")
+        return im
 
     def streamoff(self):
         """
         Stops camera streaming
         """
+        self._display_lock.acquire()
+        self.__display = False
+        self._display_lock.release()
         self.send(msg_type="ST0")
         time.sleep(1.0)
         self.tcp_video_int.stop_receiver()
@@ -686,29 +747,14 @@ class Cognifly:
 
 
 if __name__ == '__main__':
+    pass
+
     cf = Cognifly(drone_hostname="moderna.local")
 
-    # take off:
-    cf.takeoff()
+    cf.streamon(fps=30)
 
-    # display the stream:
-    cf.stream()
-    time.sleep(10.0)
+    time.sleep(120.0)
 
-    # stop the stream:
     cf.streamoff()
-    time.sleep(5.0)
-
-    # turn the stream on with no display:
-    cf.streamon()
-
-    # retrieve a frame for processing:
-    numpy_image = cf.get_frame()
-
-    # turn the stream off:
-    cf.streamon()
-
-    # land:
-    cf.land()
-
 
+    time.sleep(10.0)
